package nl.iobyte.themepark.api.database.objects;

import nl.iobyte.themepark.scheduler.ThemeParkScheduler;
import nl.iobyte.themepark.scheduler.Task;
import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

public abstract class Database {

    public abstract Connection getConnection() throws SQLException;

    public abstract void closeConnection();

    public Task<Boolean> executeAsync(String query, Map<Integer, Object> objects) {
        Task<Boolean> task = new Task<>();
        ThemeParkScheduler.runAsync(() -> task.success(execute(
                query,
                objects
        )));

        return task;
    }

    public boolean execute(String query, Map<Integer, Object> objects) {
        if(query == null || query.isEmpty())
            return false;

        boolean b = false;
        try(Connection conn = getConnection()) {
            PreparedStatement statement = conn.prepareStatement(query);
            statement.closeOnCompletion();
            if(objects != null)
                for(Map.Entry<Integer, Object> entry : objects.entrySet())
                    statement.setObject(entry.getKey(), entry.getValue());

            b = statement.execute();
        } catch(Exception e) {
            System.out.println("Query: "+query);
            System.out.println("Map: "+objects);
            e.printStackTrace();
        }

        return b;
    }

    public Task<Integer> executeUpdateAsync(String query, Map<Integer, Object> objects) {
        Task<Integer> task = new Task<>();
        ThemeParkScheduler.runAsync(() -> task.success(executeUpdate(
                query,
                objects
        )));

        return task;
    }

    public int executeUpdate(String query, Map<Integer, Object> objects) {
        if (query == null || query.isEmpty())
            return 0;

        int i = 0;
        try(Connection conn = getConnection()) {
            PreparedStatement statement = conn.prepareStatement(query);
            statement.closeOnCompletion();
            for (Map.Entry<Integer, Object> entry : objects.entrySet())
                statement.setObject(entry.getKey(), entry.getValue());

            i = statement.executeUpdate();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return i;
    }

    public Task<ArrayList<Map<String, Object>>> executeQueryAsync(String query, Map<Integer, Object> objects) {
        Task<ArrayList<Map<String, Object>>> task = new Task<>();
        ThemeParkScheduler.runAsync(() -> task.success(executeQuery(
                query,
                objects
        )));

        return task;
    }

    public ArrayList<Map<String, Object>> executeQuery(String query, Map<Integer, Object> objects) {
        if (query == null || query.isEmpty())
            return null;

        ArrayList<Map<String, Object>> set = null;
        try(Connection conn = getConnection()) {
            PreparedStatement statement = conn.prepareStatement(query);
            statement.closeOnCompletion();
            if (objects != null)
                for (Map.Entry<Integer, Object> entry : objects.entrySet())
                    statement.setObject(entry.getKey(), entry.getValue());

            set = new ArrayList<>();

            ResultSet result = statement.executeQuery();
            ResultSetMetaData metadata = result.getMetaData();
            int columnCount = metadata.getColumnCount();
            while(result.next()) {
                Map<String, Object> map = new HashMap<>();
                for(int i = 1; i <= columnCount; i++)
                    map.put(metadata.getColumnName(i), result.getObject(i));

                set.add(map);
            }

            result.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return set;
    }

}

