package nl.iobyte.themepark.api.sign;

import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.attraction.objects.Attraction;
import nl.iobyte.themepark.api.config.enums.StorageLocation;
import nl.iobyte.themepark.api.sign.objects.StatusSign;
import nl.iobyte.themepark.utils.LocationUtil;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.block.Sign;
import java.util.ArrayList;
import java.util.HashMap;

public class SignManager {

    private final HashMap<Attraction, ArrayList<StatusSign>> signs = new HashMap<>();

    public HashMap<Attraction, ArrayList<StatusSign>> getSigns() {
        return signs;
    }

    public boolean addSign(StatusSign sign) {
        if(sign == null || sign.getLocation() == null)
            return false;

        if(!(sign.getLocation().getBlock().getState() instanceof Sign))
            return false;

        if(signs.containsKey(sign.getAttraction())) {
            signs.get(sign.getAttraction()).add(sign);
            return true;
        }

        ArrayList<StatusSign> array = new ArrayList<>();
        array.add(sign);
        signs.put(sign.getAttraction(), array);
        return true;
    }

    public boolean hasSigns(Attraction attraction) {
        if(attraction == null)
            return false;

        return signs.containsKey(attraction) && (signs.get(attraction).size() > 0);
    }

    public boolean isSign(Attraction attraction, Location location) {
        if(attraction == null || location == null)
            return false;

        ArrayList<StatusSign> array = signs.get(attraction);
        if(array == null)
            return false;

        String loc = LocationUtil.toString(location);

        for(StatusSign sign : array) {
            String str = LocationUtil.toString(sign.getLocation());
            if(loc.equals(str))
                return true;
        }

        return false;
    }

    public void update(Attraction attraction) {
        if(attraction == null)
            return;

        if(!hasSigns(attraction))
            return;

        for(StatusSign sign : signs.get(attraction))
            sign.update();
    }

    public void remove(Attraction attraction) {
        if(attraction == null || !signs.containsKey(attraction))
            return;

        for(StatusSign sign : signs.get(attraction))
            sign.getLocation().getBlock().setType(Material.AIR);

        signs.remove(attraction);
        ThemePark.getInstance().getAPI().getConfigurationManager().set(StorageLocation.SIGN_DATA, "signs."+attraction.getID(), null);
    }

    public boolean removeSign(StatusSign sign) {
        if(sign == null || sign.getLocation() == null || sign.getAttraction() == null)
            return false;

        if(!hasSigns(sign.getAttraction()))
            return false;

        String str2;
        Location loc1 = sign.getLocation();
        String str1 = LocationUtil.toString(loc1);
        ArrayList<StatusSign> array = signs.get(sign.getAttraction());
        boolean b = false;
        for(StatusSign s : new ArrayList<>(array)) {
            Location loc2 = s.getLocation();
            if(loc1 == null || loc2 == null)
                continue;

            str2 = LocationUtil.toString(loc2);
            if(!str1.equals(str2))
                continue;

            array.remove(s);
            b = true;
            break;
        }

        if(array.isEmpty()) {
            signs.remove(sign.getAttraction());
            ThemePark.getInstance().getAPI().getConfigurationManager().set(StorageLocation.SIGN_DATA, "signs."+sign.getAttraction().getID(), null);
            return b;
        }

        signs.put(sign.getAttraction(), array);
        return b;
    }

}
