package nl.iobyte.themepark.commands.subcommands.region;

import com.google.common.collect.Iterables;
import nl.iobyte.commandapi.arguments.number.IntegerArgument;
import nl.iobyte.commandapi.interfaces.ICommandExecutor;
import nl.iobyte.commandapi.objects.SubCommand;
import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.attraction.objects.Region;
import nl.iobyte.themepark.api.message.Text;
import java.util.Collection;
import java.util.List;

public class RegionListCommand extends SubCommand {

    public RegionListCommand(String cmd)  {
        super("themepark.admin", "region", "list");

        addSyntax("/"+cmd+" region list");

        addSyntax("/"+cmd+" region list <page>")
                .addArgument(new IntegerArgument());
    }

    public void onCommand(ICommandExecutor  sender, List<Object> list, int i) {
        Collection<Region> regions = ThemePark.getInstance().getAPI().getAttractionService().getRegions().values();
        if(regions.size() <= 5) {
            sendSinglePage(sender, regions);
        } else {
            sendMultiPage(sender, regions, list, i);
        }
    }

    private void sendSinglePage(ICommandExecutor sender, Collection<Region> regions) {
        sender.sendMessage(Text.color("&f&l>==== &6&lThemePark &l&f====<"));
        if(regions.isEmpty()) {
            sender.sendMessage(Text.color("&6No regions found"));
        } else {
            for (Region region : regions)
                sender.sendMessage(Text.color("&6ID: &f" + region.getID() + " &6Name: &r" + region.getName()));
        }

        sender.sendMessage(Text.color("&f&l>==== &6&lThemePark &l&f====<"));
    }

    private void sendMultiPage(ICommandExecutor sender, Collection<Region> regions, List<Object> list, int i) {
        int page = 1;
        int pages = (int) Math.ceil(((double) regions.size()) / 5);
        if(i == 1)
            page = (Integer) list.get(0);

        if(page < 1 || page > pages) {
            sender.sendMessage(Text.color("&6&lThemePark &f➢ Page &6"+page+" &fdoesn't exist"));
            return;
        }

        i = 0;
        for(List<Region> lr : Iterables.partition(regions, 5)) {
            i++;
            if(i != page)
                continue;

            sender.sendMessage(Text.color("&f&l>==== &6&lThemePark &f(&6"+(page + 1)+"&f/&6"+pages+"&f) &l&f====<"));
            for(Region region : lr)
                sender.sendMessage(Text.color("&6ID: &f"+region.getID()+" &6Name: &r"+region.getName()));

            sender.sendMessage(Text.color("&f&l>==== &6&lThemePark &l&f====<"));
            break;
        }
    }

}
