package nl.iobyte.themepark.listeners;

import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.attraction.objects.Region;
import nl.iobyte.themepark.api.config.enums.StorageLocation;
import nl.iobyte.themepark.api.event.region.RegionCreateEvent;
import nl.iobyte.themepark.api.event.region.RegionMaterialChangeEvent;
import nl.iobyte.themepark.api.event.region.RegionNameChangeEvent;
import nl.iobyte.themepark.api.event.region.RegionRemoveEvent;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import java.util.HashMap;

public class RegionListener implements Listener {

    @EventHandler
    public void onCreate(RegionCreateEvent e) {
        Region region = e.getRegion();

        //Add Region to config
        ThemePark.getInstance().getAPI().getConfigurationManager().set(
                StorageLocation.REGIONS,
                "regions."+region.getID()+".name",
                region.getName()
        );
        ThemePark.getInstance().getAPI().getConfigurationManager().set(
                StorageLocation.REGIONS,
                "regions."+region.getID()+".material",
                region.getMaterial().toString()
        );

        //Update Menu
        ThemePark.getInstance().getAPI().getMenuService().getStatusMenu().load();

        //Update MySQL Database
        ThemePark.getInstance().getAPI().getDatabaseService().executeAsync(
                "remote",
                "INSERT IGNORE INTO regions(id, name) VALUES (?,?)",
                new HashMap<>() {{
                    put(1, region.getID());
                    put(2, region.getName());
                }}
        );
    }

    @EventHandler
    public void onNameChange(RegionNameChangeEvent e) {
        Region region = e.getRegion();

        //Change Region name in config
        ThemePark.getInstance().getAPI().getConfigurationManager().set(
                StorageLocation.REGIONS,
                "regions."+region.getID()+".name",
                region.getName()
        );

        //Update Menu
        ThemePark.getInstance().getAPI().getMenuService().getStatusMenu().updateRegion(region);

        //Update MySQL Database
        ThemePark.getInstance().getAPI().getDatabaseService().executeAsync(
                "remote",
                "UPDATE regions SET name=? WHERE id=?",
                new HashMap<>() {{
                    put(1, region.getName());
                    put(2, region.getID());
                }}
        );
    }

    @EventHandler
    public void onNameChange(RegionMaterialChangeEvent e) {
        Region region = e.getRegion();

        //Change Region name in config
        ThemePark.getInstance().getAPI().getConfigurationManager().set(
                StorageLocation.REGIONS,
                "regions."+region.getID()+".material",
                region.getMaterial().toString()
        );

        //Update Menu
        ThemePark.getInstance().getAPI().getMenuService().getStatusMenu().updateRegion(region);
    }

    @EventHandler
    public void onRemove(RegionRemoveEvent e) {
        Region region = e.getRegion();
        region.getConfiguration().delete();

        //Remove Region from config
        ThemePark.getInstance().getAPI().getConfigurationManager().set(
                StorageLocation.REGIONS,
                "regions."+region.getID(),
               null
        );

        //Update Menu
        ThemePark.getInstance().getAPI().getMenuService().getStatusMenu().load();

        //Update MySQL Database
        ThemePark.getInstance().getAPI().getDatabaseService().executeAsync(
                "remote",
                "DELETE regions, attractions, ridecounts FROM regions LEFT JOIN attractions ON regions.id=attractions.region_id LEFT JOIN ridecounts ON attractions.id=ridecounts.attraction_id WHERE regions.id=?",
                new HashMap<>() {{
                    put(1, region.getID());
                }}
        );
    }

}
