package nl.iobyte.themepark.scheduler;

import java.util.function.Consumer;

public class Task<T> {

    private Consumer<String> whenFails;
    private Consumer<T> whenSuccessful;

    /**
     * Set what to do if task fails
     * @param whenFails BiConsumer with ErrorCode and Message
     */
    public void setWhenFails(Consumer<String> whenFails) {
        this.whenFails = whenFails;
    }

    /**
     * Set what to do if task is successful
     * @param whenSuccessful Consumer with the result
     */
    public void setWhenSuccessful(Consumer<T> whenSuccessful) {
        this.whenSuccessful = whenSuccessful;
    }

    /**
     * Task was a success
     * @param data Return data
     */
    public void success(T data) {
        if(whenSuccessful != null)
            whenSuccessful.accept(data);
    }

    /**
     * Task failed without message
     */
    public void fail() {
        if (whenFails != null) whenFails.accept("No message provided");
    }

    /**
     * Task failed with message
     * @param message Error message
     */
    public void fail(String message) {
        if (whenFails != null) whenFails.accept(message);
    }

}
